// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <suplib/str.h>

#include "ParTxt.h"
#include "Par.h"
#include "ParFile.h"

int ParTxt::restore=0;

ParTxt::~ParTxt( ) {

  buffer = NULL;

  if ( buffer_argv )
    str_tokenize_free( buffer_argv, restore );
  
}

ParTxt::ParTxt( char* str ) {

  buffer_argv = NULL;
  num_tokens = 0;

  if ( 0 == str )
    return;

  buffer = str;

  try {
    init_ParTxt( str );
  } catch( ParFileException& pfe ) {
    throw;
  }

}

char* ParTxt::get_token( size_t num ) {

  if ( NULL == buffer_argv ) {
    char msg[ 128 ];
    const char* format = "ParTxt::get_token( %d )\n";
    sprintf( msg, format, num );
    throw ParFileException( msg );
  }

  return buffer_argv[ num ];

}

void ParTxt::init_ParTxt( char* str ) {

  buffer_argv = NULL;
  num_tokens = 0;

  if ( ParFile::is_comment( str ) )
    return;

  buffer = str;

  char delimit[ 2 ] = { Par::DELIMIT, '\0' };
  
  num_tokens = str_tokenize( str, delimit, &buffer_argv, 0, restore );

  if ( Par::NUMTOKENS != num_tokens ) {
    char msg[ 256 ];
    const char* format = 
      "ParTxt::init_ParTxt( %s ) : Expected %d tokens, but parsed %d "
      "number of tokens";
    sprintf( msg, format, str, Par::NUMTOKENS, num_tokens );
    throw ParFileException( msg );
  }

  for ( int ii = 0; ii < num_tokens; ii++ ) {
    if ( ii != num_tokens - 1 )
      unquote( buffer_argv[ ii ] );
    str_trim( buffer_argv[ ii ] );
    str_prune( buffer_argv[ ii ] );
  }

  try {

    verify_mode( buffer_argv[ Par::PARMODE ] );

  } catch( ParFileException& pfe ) {
    throw;
  }

}

void ParTxt::print( ostream& os ) {

  os << buffer;
  if ( 0 == num_tokens )
    return;

  for ( int i = 0; i < num_tokens; i++ )
    os << "buffer_argv[" << i << "] = `" << buffer_argv[ i ] << "'\n";

}

/**
 * Check to see if `mode' is one of the followig options: a, b, h, H, l
 */
void ParTxt::verify_mode( const char* str ) {
  int c;
  while( (c=*str++) )
    switch( c ) {
    case Par::AUTOS:
    case Par::BATCH:
    case Par::hIDDEN:
    case Par::HIDDEN:
    case Par::QUERRY:
    case Par::LEARN:
      break;
    default:
        throw ParFileException( "ParTxt::verify_mode( " + string(str) +
			      " ) : Incorrect mode\n" );
    }

  return;

}
