#ifndef STRINGPAR_H
#define STRINGPAR_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "Par.h"

/**
 * A string parameter.
 * The class is reponsible to make sure that
 * the parameter can only take a string value.
 */
class StringPar : public Par {

public:

  ~StringPar( ) { }

  StringPar( ) : Par( ) { }

  /**
   * The copy constructor.
   */
  StringPar( const StringPar& par ) : Par( par ) { }

  // constructor
  StringPar( ParTxt& par ) ;

  void pgetstr( char result[], size_t size ) const  {
    strncpy( result, parameter[ PARVALUE ].c_str( ), size );
    result[ size ] = '\0';
  }

  string pgetstring( ) const  {
    return parameter[ PARVALUE ];
  }

  /**
   * To output the parameter of the form: name,type,mode,"val",min,max,prompt
   */
  void print( ostream& os ) const;

  void set_val( const string& str ) ;

private:

  void between_limits( const string& t ) const
    ;

  void validate( const string& ) const  {
    return;
  }
   
};

#endif
