#include <stdlib.h>
#include <iostream>

#include <testlib/testlib.h>

#include <paramxx/ParFile.h>

static void test_boolean( ParFile& pf ) {

    bool abool = pf.pgetb( "aboolean" );

    bool anotherbool;
    pf.pget( "aboolean", anotherbool );

    TEST( "boolean", abool, anotherbool );

}

static void test_double( ParFile& pf ) {

    double adouble = pf.pgetd( "areal" );

    double anotherdouble;
    pf.pget( "areal", anotherdouble );

    TEST( "double", adouble, anotherdouble );
}

static void test_int( ParFile& pf ) {

    int anint = pf.pgeti( "anint" );

    int anotherint;

    pf.pget( "anint", anotherint );

    TEST( "integer", anint, anotherint );
}

static void test_long( ParFile& pf ) {

    long along = pf.pgetl( "along" );

    long anotherlong;
    pf.pget( "along", anotherlong );

    TEST( "long", along, anotherlong );
}

static void test_string( ParFile& pf ) {


    string astring = pf.pgetstring( "astr" );
    
    string anotherstring;
    pf.pget( "astr", anotherstring );

    TESTSTRING( "string", astring.c_str( ), anotherstring.c_str( ) );
}

int main( int argc, char** argv ) {

  START( "Testing ParFile with indirrection" );

  try {
    ParFile pf( argc, argv );

    char str[ 256 ];
    pf.pgetstr( "foo", str, 256 );

    test_boolean( pf );

    test_int( pf );

    test_long( pf );

    test_string( pf );

    test_double( pf );

  } catch ( ParFileException& pfe ) {
    cerr << pfe;
    SUMMARY();
    return EXIT_FAILURE;
  } catch ( Exception& e ) {
    cerr << e;
    SUMMARY();
    return EXIT_FAILURE;
  }

  SUMMARY();

  return FAILED() ? EXIT_FAILURE : EXIT_SUCCESS;
}
