#include <cstdlib>
#include <limits.h>
#include <testlib/testlib.h>

#include <paramxx/RealPar.h>

#include "test.hpp"

#define MYLENGTH 128
static char mystr[ MYLENGTH ];

static int testReal( bool printIt, int num ) {

  try {

    sprintf( mystr,
	     "anno_weight,r,a,4,0,%d,\"blah blah bhal\"", INT_MAX );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

    if ( printIt )
      cout << realPar << endl;

    for ( int i = 0; i < num; i++ ) {
      int val = rand( ) % RAND_MAX;
      char tmp[ MYLENGTH ];
      sprintf( tmp, "%d", val );
      realPar.set_val( tmp );
      TEST_FP( "RealPar set_val and pgeti", realPar.pgetd( ), val );
    }

    return EXIT_SUCCESS;

  } catch( ParFileException& pfe ) {

    if ( printIt )
      cerr << pfe;
    return EXIT_FAILURE;

  } catch ( Exception & e ) {

    if ( printIt )
      cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

}

static int illegal_min( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,12345a,,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

  } catch( ParFileException& pfe ) {

    TEST( "RealPar Illegal_min", EXIT_SUCCESS, EXIT_SUCCESS );
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  TEST( "RealPar Illegal_min", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int illegal_max( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,12345a,,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

  } catch( ParFileException& pfe ) {

    TEST( "RealPar Illegal_max", EXIT_SUCCESS, EXIT_SUCCESS );
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  TEST( "Illegal_max", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_bool_type( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,,,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

    bool foo = realPar.pgetb( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "RealPar not_bool_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "RealPar not_bool_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_int_type( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,,,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

    int foo = realPar.pgeti( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "RealPar not_int_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "RealPar not_int_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_str_type( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,,,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

    realPar.pgetstr( mystr, MYLENGTH );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "RealPar not_str_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "RealPar not_str_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_string_type( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,,,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

    string foo = realPar.pgetstring( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "RealPar not_string_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "RealPar not_string_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int min_biggerthen_val( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,6,,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "RealPar min_biggerthen_val( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "RealPar not_string_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int max_lessthen_val( bool printIt ) {

  try {

    strcpy( mystr, "anno_weight,r,a,4,,0,\"foo\"" );
    ParTxt parTxt( mystr );
    RealPar realPar( parTxt );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "RealPar max_lessthen_val( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "RealPar not_string_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

int testRealPar( bool printIt, int num ) {

  if ( EXIT_FAILURE == testReal( printIt, num ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == illegal_min( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == illegal_max( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_bool_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_int_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_str_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_string_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == min_biggerthen_val( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == max_lessthen_val( printIt ) )
    return EXIT_FAILURE;

  return EXIT_SUCCESS;

}

